<?php
/**
 * Skrill Order
 *
 * The Class for Process Skrill Payment Gateways
 * Copyright (c) Skrill
 *
 * @class   SkrillOrder
 * @package Skrill/Classes
 * @located at /controllers
 */
class SkrillOrder extends skrillorder_parent
{
    /**
     * Render template for order
     *
     * @return string template path
     */
    public function render()
    {
        oxRegistry::getSession()->deleteVariable('skrill_sid');
        $payment = $this->getPayment();
        $user = $this->getUser();
        $config = $this->getViewConfig();
        $userCountryCode = SkrillOxCountry::getUserCountryIso3();
        $paymentMethod = SkrillCore::getFrontendLogoHtmlById($payment->oxpayments__oxid->value, $userCountryCode);
        if (empty($paymentMethod)) {
            $paymentMethod = $payment->oxpayments__oxdesc->value;
        }
        $payment->oxpayments__oxdesc->value = $paymentMethod;
        return parent::render();
    }

    /**
     * (Overwritten) Get next step
     *
     * @param  integer $orderState order state
     * @return string
     */
    protected function _getNextStep($orderState)
    {
        oxRegistry::getSession()->setVariable('sess_challenge', oxUtilsObject::getInstance()->generateUID());
        $order = SkrillOxOrder::refreshOrder();

        if ($orderState === oxOrder::ORDER_STATE_ORDEREXISTS
            && !$order->isSkrillPaymentDone()
            && !$order->isLoaded()) {
            list($order, $orderState) = $this->_recreateOrder($order);
        }

        if (!$order->isSkrillOrder()) {
            return parent::_getNextStep($orderState);
        }

        //check for one page checkout => call orderdirect-service
        $paymentId   = $this->getPayment()->getId();
        $userId = oxRegistry::getSession()->getUser()->oxuser__oxid->value;
        $currentSession = oxRegistry::getSession()->getVariable('sess_challenge');
        $existingOrder = $this->existingOrder($currentSession, $userId);
        if (isset($existingOrder) && $existingOrder['OXTRANSSTATUS'] == "OK") {
            $orderState = $existingOrder['OXTRANSSTATUS'];
            $this->updateDataExistingOrder($order, $existingOrder);
            oxRegistry::getSession()->setVariable('sess_challenge', $existingOrder['OXID']);
        } else {
            $this->createOrderBeforePayment($order);
        }

        if ($orderState === oxOrder::ORDER_STATE_OK ||
            $orderState === oxOrder::ORDER_STATE_MAILINGERROR ||
            !$order->isSkrillPaymentDone()
        ) {
            $paymentId = str_replace("skrill_", "", $paymentId);
            $paymentId = ucwords(strtolower($paymentId));
            return 'SkrillPayment' . $paymentId;
        }

        return parent::_getNextStep($orderState);
    }

    /**
     * function to create order before redirect to payment widget
     * @param  object $order
     * @return void
     */
    public function createOrderBeforePayment($order)
    {
        $basket = oxRegistry::getSession()->getBasket();
        $user = oxRegistry::getSession()->getUser();
        
        $order->finalizeOrder($basket, $user, false, true);
    }

    /**
     * Recreate order
     *
     * @param  oxOrder $order order object
     * @return array
     */
    protected function _recreateOrder(oxOrder $order)
    {
        $orderNumber = $order->oxorder__oxordernr->value;

        // create order number reservation
        $orderNumberReservation = oxNew('SkrillOrderNumberReservation');
        $reservationKey         = SkrillOrderNumberReservation::getReservationKey($orderNumber);
        if (!$orderNumberReservation->load($reservationKey)) {
            $orderNumberReservation->setId($reservationKey);
            $orderNumberReservation->save();
        }

        $order->delete();
        $newOrder   = oxNew('skrilloxorder');
        $newOrder->oxorder__oxordernr = new oxField($orderNumber, oxField::T_RAW);
        $orderState = $newOrder->finalizeOrder($this->getBasket(), $this->getUser(), true);
        $this->getUser()->onOrderExecute($this->getBasket(), $orderState);

        // delete order number reservation
        $orderNumberReservation->delete();

        return array($newOrder, $orderState);
    }

    /**
     * Payment response
     *
     * @return string
     */
    public function paymentResponse()
    {
        SkrillLogger::set()->info('Skrill - start process return url');
        SkrillLogger::set()->info('Skrill - post parameters: '.json_encode($_REQUEST));
        
        $transactionId = oxRegistry::getConfig()->getRequestParameter('transaction_id');
        $oxid = oxRegistry::getConfig()->getRequestParameter('oxid');

        for ($i=1; $i<= 5; $i++) {
            SkrillLogger::set()->info('Skrill - get skrill payment log data: '.$i);

            $paymentResponse = SkrillCore::getPaymentResponseByTransactionId($transactionId);

            if (!empty($paymentResponse)) {
                break;
            }
            sleep(1);
        }

        SkrillLogger::set()->info('Skrill - get payment response from database: '.json_encode($paymentResponse));

        if (empty($paymentResponse)) {
            SkrillLogger::set()->info('Skrill - status url late');
            try {
                $paymentResponse = SkrillCore::getPaymentResponseFromGateway($transactionId);
            } catch (Exception $e) {
                SkrillLogger::set()->info('Skrill - error when get payment status '. $e->getMessage());
            }

            SkrillLogger::set()->info('Skrill - get payment response with send request to skrill: '.
                json_encode($paymentResponse));

            $isPaymentResponseValid = SkrillCore::isPaymentResponseValid($paymentResponse);
            if (!$isPaymentResponseValid) {
                SkrillLogger::set()->info('Skrill - fraud detected');
                $paymentResponse['status'] = SkrillCore::FRAUDSTATUS;
            }

            SkrillCore::updateExistingOrderStatus($oxid, $paymentResponse);
            SkrillCore::addPaymentLog($oxid, $paymentResponse);
        }

        if ($paymentResponse['status'] == SkrillCore::PROCESSEDSTATUS ||
            $paymentResponse['status'] == SkrillCore::PENDINGSTATUS
        ) {
            SkrillLogger::set()->info('Skrill - payment successfull');

            // prevent basketitem isBuyable validation
            // use of basket between order and thankyou views causes errors, when buying last item in stock
            oxRegistry::getConfig()->setConfigParam('skrill__prevent_recalculate', true);
            oxRegistry::getSession()->getBasket()->afterUpdate();

            $orderState = oxOrder::ORDER_STATE_OK;
            $skrillOxOrder = oxNew('skrilloxorder');
            $skrillOxOrder->load($oxid);
            $orderState = $skrillOxOrder->skrillSendEmails();
            oxRegistry::getSession()->setVariable('skrill_order_status', 'success');
            oxRegistry::getSession()->deleteVariable('skrill_order_status');
            return parent::_getNextStep($orderState);
        } elseif ($paymentResponse['status'] == SkrillCore::FRAUDSTATUS) {
            SkrillLogger::set()->info('Skrill - payment fraud');
            $this->redirectError('FRAUD_DETECTED');
        } else {
            SkrillLogger::set()->info('Skrill - payment failed with reason: '. $paymentResponse['failed_reason_code']);
            $errorStatus = SkrillCore::getSkrillErrorMapping($paymentResponse['failed_reason_code']);
            $this->redirectError($errorStatus);
        }
    }

    /**
     * Redirect when error payment process
     *
     * @param  string $errorIdentifier error identifier
     */
    public function redirectError($errorIdentifier)
    {
        SkrillLogger::set()->info('Skrill - redirect error: '. $errorIdentifier);
        oxRegistry::get('oxUtilsView')->addErrorToDisplay($errorIdentifier);
        $redirectUrl = oxRegistry::getConfig()->getSslShopUrl() . 'index.php?cl=order';
        oxRegistry::getUtils()->redirect($redirectUrl);
    }

    /**
     * get existing order data
     * @param  string $sessionID
     * @param  string $userID
     * @return array|boolean
     */
    public function existingOrder($sessionID, $userID = false)
    {
        if ($userID) {
            $lastOrderID = oxDb::getDb(oxDb::FETCH_MODE_ASSOC)->getRow(
                "SELECT max(OXORDERNR) as OXORDERNR FROM oxorder WHERE OXUSERID = '" .$userID. "'
                AND DATE_FORMAT(OXORDERDATE,'%Y-%m-%d') = '".date('Y-m-d')."'"
            );
            $orderData = oxDb::getDb(oxDb::FETCH_MODE_ASSOC)->getRow(
                "SELECT * FROM oxorder WHERE OXORDERNR = '" .$lastOrderID['OXORDERNR']. "'"
            );
        } else {
            $orderData = oxDb::getDb(oxDb::FETCH_MODE_ASSOC)->getRow(
                "SELECT * FROM oxorder WHERE OXID = '" .$sessionID. "'"
            );
        }

        if (!empty($orderData)) {
            return $orderData;
        } else {
            return false;
        }
    }

    /**
     * update data existing order
     * @param  object $order
     * @param  array $existingOrder
     * @return void
     */
    public function updateDataExistingOrder($order, $existingOrder)
    {
        $oxid = $existingOrder['OXID']; //oxid of existing order
        $oOrder = oxNew("oxorder");
        $oOrder->load($oxid);
        $oOrder->oxorder__oxbillcompany->setValue($order->oxorder__oxbillcompany->value);
        $oOrder->oxorder__oxbillemail->setValue($order->oxorder__oxbillemail->value);
        $oOrder->oxorder__oxbillfname->setValue($order->oxorder__oxbillfname->value);
        $oOrder->oxorder__oxbilllname->setValue($order->oxorder__oxbilllname->value);
        $oOrder->oxorder__oxbillstreet->setValue($order->oxorder__oxbillstreet->value);
        $oOrder->oxorder__oxbillstreetnr->setValue($order->oxorder__oxbillstreetnr->value);
        $oOrder->oxorder__oxbilladdinfo->setValue($order->oxorder__oxbilladdinfo->value);
        $oOrder->oxorder__oxbillustid->setValue($order->oxorder__oxbillustid->value);
        $oOrder->oxorder__oxbillcity->setValue($order->oxorder__oxbillcity->value);
        $oOrder->oxorder__oxbillcountryid->setValue($order->oxorder__oxbillcountryid->value);
        $oOrder->oxorder__oxbillstateid->setValue($order->oxorder__oxbillstateid->value);
        $oOrder->oxorder__oxbillzip->setValue($order->oxorder__oxbillzip->value);
        $oOrder->oxorder__oxbillfon->setValue($order->oxorder__oxbillfon->value);
        $oOrder->oxorder__oxbillfax->setValue($order->oxorder__oxbillfax->value);
        $oOrder->oxorder__oxbillsal->setValue($order->oxorder__oxbillsal->value);
        $oOrder->oxorder__oxdelcompany->setValue($order->oxorder__oxdelcompany->value);
        $oOrder->oxorder__oxdelfname->setValue($order->oxorder__oxdelfname->value);
        $oOrder->oxorder__oxdellname->setValue($order->oxorder__oxdellname->value);
        $oOrder->oxorder__oxpaymentid->setValue($order->oxorder__oxpaymentid->value);
        $oOrder->oxorder__oxpaymenttype->setValue($order->oxorder__oxpaymenttype->value);
        $oOrder->oxorder__oxtotalnetsum->setValue($order->oxorder__oxtotalnetsum->value);
        $oOrder->oxorder__oxtotalbrutsum->setValue($order->oxorder__oxtotalbrutsum->value);
        $oOrder->oxorder__oxtotalordersum->setValue($order->oxorder__oxtotalordersum->value);
        $oOrder->oxorder__oxartvat1->setValue($order->oxorder__oxartvat1->value);
        $oOrder->oxorder__oxartvatprice1->setValue($order->oxorder__oxartvatprice1->value);
        $oOrder->oxorder__oxartvat2->setValue($order->oxorder__oxartvat2->value);
        $oOrder->oxorder__oxartvatprice2->setValue($order->oxorder__oxartvatprice2->value);
        $oOrder->oxorder__oxdelcost->setValue($order->oxorder__oxdelcost->value);
        $oOrder->oxorder__oxdelvat->setValue($order->oxorder__oxdelvat->value);
        $oOrder->oxorder__oxpaycost->setValue($order->oxorder__oxpaycost->value);
        $oOrder->oxorder__oxpayvat->setValue($order->oxorder__oxpayvat->value);
        $oOrder->oxorder__oxwrapcost->setValue($order->oxorder__oxwrapcost->value);
        $oOrder->oxorder__oxwrapvat->setValue($order->oxorder__oxwrapvat->value);
        $oOrder->oxorder__oxgiftcardcost->setValue($order->oxorder__oxgiftcardcost->value);
        $oOrder->oxorder__oxgiftcardvat->setValue($order->oxorder__oxgiftcardvat->value);
        $oOrder->oxorder__oxcardid->setValue($order->oxorder__oxcardid->value);
        $oOrder->oxorder__oxcardtext->setValue($order->oxorder__oxcardtext->value);
        $oOrder->oxorder__oxdiscount->setValue($order->oxorder__oxdiscount->value);
        $oOrder->oxorder__oxexport->setValue($order->oxorder__oxexport->value);
        $oOrder->oxorder__oxbillnr->setValue($order->oxorder__oxbillnr->value);
        $oOrder->oxorder__oxremark->setValue($order->oxorder__oxremark->value);
        $oOrder->oxorder__oxvoucherdiscount->setValue($order->oxorder__oxvoucherdiscount->value);
        $oOrder->oxorder__oxcurrency->setValue($order->oxorder__oxcurrency->value);
        $oOrder->oxorder__oxcurrate->setValue($order->oxorder__oxcurrate->value);
        $oOrder->save();
    }
}
